from utils import getnibdata, mcao_ppms, mcao_satppmidx,\
                  nrats, rats, mcao_masks,\
                  pickleload, pickledump,\
                  get_num_model_params, get_log_likelihood, calcr2, get_voxs
                  
import numpy as np
from matplotlib import pyplot as plt
from scipy import stats
import os.path as op
from sys import exit
from itertools import product

ppms = mcao_ppms
satppmidx = mcao_satppmidx
ratnums = np.arange(nrats)

#%% extract spectra
precomputed = pickleload('mcao_precomputed_mean_spectra.pickle')

#%% plot
#plt.close('all')

r = 1
pool = 4
lda = 'lda1'

roi_mean_spectra = precomputed[f'{r}-{pool}-{lda}']

#
#Subplots for assessing general fit [over entire mask]
#
f, axarr = plt.subplots(2,3)
axarr = axarr.flatten()

maskname = 'mask'
#roi_mean_spectra = precomputed[f'{r}-{pool}-{lda}']
acqs, fits3, wfit_means, lda_means, residuals = precomputed[f'{r}-3-{lda}'][maskname]
acqs, fits4, wfit_means, lda_means, residuals = precomputed[f'{r}-4-{lda}'][maskname]

clrs = ['red','olivedrab']
lines = ['-', '-']

r2_3p = np.zeros(6)*np.nan
r2_4p = np.zeros_like(r2_3p)

r2cestidxs = list(range(1,12)) + list(range(19,40)) #+/- (4.1 to 1.2)

for i in range(nrats):
    acq = acqs.T[:,i]
    fit3 = fits3.T[:,i]
    fit4 = fits4.T[:,i]
    
    r2_3p[i] = calcr2(acq, fit3, r2cestidxs)
    r2_4p[i] = calcr2(acq, fit4, r2cestidxs)
    ax = axarr[i]
    ax.plot(ppms, fit3, lines[0], c=clrs[0], lw=1, label='3 pool fit')
    ax.plot(ppms, fit4, lines[1], c=clrs[1], lw=1, label='4 pool fit')
    ax.plot(ppms, acq, 'kd', ms=2, label='acquired data')
    #ax.plot(ppms, wfit_means.T[:,i], 'r-', lw=1, label=lda)    

    ax.plot(ppms, fit3 - acq, '--', c=clrs[0], lw=1, label='3 pool residual')
    ax.plot(ppms, fit4 - acq, '--', c=clrs[1], lw=1, label='4 pool residual')
    ax.plot([ppms[0], ppms[-1]], np.array([0, 0]), 'k:', lw=1)    
    
    ax.set_xlim(-4.5, 4.5)
    #ax.set_ylim(-0.1, 1.1)
    #ax.set_ylim(0.75, 1.01)
    
    #for poolmarker in [3.5,2,-1.6,-3.5]:
    #    ax.plot([poolmarker]*2,[-1,2], 'm:',lw=0.5)
    
    if i >= 3:
        ax.set_xlabel('offset frequency (ppm)')
    ax.set_ylabel('Mz')
    ax.set_title(rats[i] + ' $R^2_{cest}=$' + f'{r2_3p[i]} (3 pool), {r2_4p[i]} (4 pool)')
    ax.invert_xaxis()
    #for tick in ax.get_xticklabels(): tick.set_rotation(45)
    
handles, labels = ax.get_legend_handles_labels()
f.legend(handles, labels)
f.suptitle(f'{maskname}, run {r}, {pool}pool')
   
#
#LDA [as above, entire mask]
#
clrs = ['#0023ff', '#5900ff', '#7c00ff', '#9100ff', '#c000ff', '#eb00ff', '#ff00a4']
f, ax = plt.subplots()

for i in range(nrats):
    ax.plot(ppms, lda_means[i,:], label = rats[i],
            lw = 1+0.3*i, c=clrs[i])

ax.plot([-500, 500], [0, 0], 'k-', lw=1)
ax.set_xlim([-7.5, 7.5])
#ax.set_ylim([-0.01, 0.175])

ax.plot([3.5,3.5], [-0.04, 0.07], 'k--')
ax.plot([-3.5,-3.5], [-0.04, 0.07], 'k--')

for tick in ax.get_xticklabels(): tick.set_rotation(45)

ax.set_xlabel('offset frequency (ppm)')
ax.set_ylabel('$M^0-M_z/M^0$')
ax.invert_xaxis()
ax.legend()
f.tight_layout()

#
#Subplots for different rois
#
#plt.close('all')
roiclrs = ['olivedrab', 'purple', 'crimson', 'b']
f, axarr = plt.subplots(2,3)
axarr = axarr.flatten()

for ratidx,ax in enumerate(axarr):    #A plot per rat
    for maskidx, maskname in enumerate(mcao_masks):
        acqs, fits3, wfit_means, lda_means, residuals = precomputed[f'{r}-3-{lda}'][maskname]
        acqs, fits4, wfit_means, lda_means, residuals = precomputed[f'{r}-4-{lda}'][maskname]
        
        acq = acqs.T[:,ratidx]
        fit3 = fits3.T[:,ratidx]
        fit4 = fits4.T[:,ratidx]
        
        ax.plot(ppms, fit3, '-', lw=1, label=f'3 pool {maskname} fit', c=roiclrs[maskidx])
        ax.plot(ppms, fit4, '--', lw=1, label=f'4 pool {maskname} fit', c=roiclrs[maskidx])
        ax.plot(ppms, acq, 'd', ms=2, label=f'{maskname} data', c=roiclrs[maskidx])
        
    ax.set_xlim([-4.5, 4.5])
    #ax.set_ylim(0.75, 0.98)
   
    #for poolmarker in [3.5,2,-1.6,-3.5]:
    #    ax.plot([poolmarker]*2,[-1,2], 'm:',lw=0.5)
        
    ax.set_xlabel('offset frequency (ppm)')
    ax.set_ylabel('Mz')
    ax.invert_xaxis()
    ax.set_title(f'{rats[ratidx]}')
    #for tick in ax.get_xticklabels(): tick.set_rotation(45)
    
handles, labels = ax.get_legend_handles_labels()
f.legend(handles, labels)
f.suptitle(f'run {r}, {pool}pool')


#R2 trend
#plt.close('all')
f,ax = plt.subplots()

maskname = 'mask'
r2vals = np.zeros( [5,12] ) * np.nan
for pooli, pool in enumerate([3,4,5209,516,6]): #41,520
    for runratidx, (run, ratidx) in enumerate( product(*[[1,2], range(nrats)]) ):
        if run == 2 and rats[ratidx] == 'JL615':
            continue
        
        roi_mean_spectra = precomputed[f'{run}-{pool}-{lda}']
        acq_means, fit_means, wfit_means, lda_means, residuals = roi_mean_spectra[maskname]
        
        meanacq = acq_means[ratidx,:]
        meanfit = fit_means[ratidx,:]
        R2 = np.corrcoef(meanacq, meanfit)[0,1]**2
        R2cest = np.corrcoef(meanacq[r2cestidxs], meanfit[r2cestidxs])[0,1]**2

        """
        #For further metrics
        resultsfol = f'mcao/__fit_{rat}_run{r}_{pool}pool'
        mask = f'mcao/{rat}/maskr{r}'
        #   R2adj:
        ndatapts = fslstats(mask, '-V')[0]
        nparams = get_num_model_params(resultsfol)
        R2cest_adj = 1 - (1 - R2cest)*(ndatapts - 1)/(ndatapts - nparams - 1)
        #   attempt at AIC:    
        #raw = f'mcao/{rat}/cestr{r}'     
        #r2vals[pooli, i] = 2*nparams - 2*get_log_likelihood(raw, mask, resultsfol)
        """
        #Plot this metric:
        r2vals[pooli, runratidx] = R2cest#R2cest_adj
        
ax.errorbar(['3 pool','4 pool', '5 pool (amine)', ' 5 pool (-1.6ppm)','6 pool'],
            np.nanmean(r2vals,axis=1),
            np.nanstd(r2vals, axis=1))
ax.set_xlabel('model')
ax.set_ylabel('$R^2_{cest}$')
ax.set_xticklabels(labels=ax.get_xticklabels(), rotation=45)