from utils import getnibdata, phan_ppms, phan_satppmidx, phan_ldappmidx,\
                  vial_ph, numvials, vialnums, phan_rawcest, fslstats, generate_modelfit
import numpy as np
from matplotlib import pyplot as plt
from scipy import stats

ppms = phan_ppms
ldappmidx = phan_ldappmidx
satppmidx = phan_satppmidx
rawcest = phan_rawcest

#%% extract spectra
acq_means = np.zeros([numvials, len(ppms)])

fit_means = np.zeros_like(acq_means)
residuals = np.zeros_like(acq_means)

wfit_means = np.zeros_like(acq_means)
lda_means = np.zeros_like(acq_means)

for i, vnum in enumerate(vialnums):    
    p2mask = f'phan/masks/NB{vnum}' #Raw data
    fitfol = f'phan/NB{vnum}_std'   #3 pool fit
    wfitfol = f'phan/NB{vnum}_lda'  #Fits just the water baseline

    mask = getnibdata(p2mask)
    nvox = np.count_nonzero(mask)

    raw = getnibdata(rawcest).squeeze()[mask==1] # [nvox, nppm]
    fit = getnibdata(fitfol+'/interp_modelfit').squeeze()[mask==1]
    wfit = getnibdata(wfitfol+'/interp_modelfit').squeeze()[mask==1]
    #fit = getnibdata(fitfol+'/modelfit').squeeze()[mask==1]
    #wfit = getnibdata(wfitfol+'/modelfit').squeeze()[mask==1]
    ppmoff = getnibdata(fitfol+'/mean_ppm_off')[mask==1].reshape([-1,1]) # [nvox, 1]
    #wppmoff = getnibdata(fitfol+'/mean_ppm_off')[mask==1].reshape([-1,1]) # [nvox, 1]
    
    #m0a
    m0a_fit = getnibdata(fitfol+'/mean_M0a')[mask==1].reshape([-1,1])
    m0a_w = getnibdata(wfitfol+'/mean_M0a')[mask==1].reshape([-1,1])
    m0a_data = np.mean(raw[:, satppmidx ], axis=1).reshape([-1,1])
    
    #normalise spect
    acq_z = raw / m0a_fit
    fit_z = fit / m0a_fit
    wfit_z = wfit / m0a_w
    
    #correct for each voxel's offset
    for vox in range(nvox):
        acq_z[vox, :] = np.interp(ppms, ppms - ppmoff[vox], acq_z[vox, :])
        #fit_z[vox, :] = np.interp(ppms, ppms - ppmoff[vox], fit_z[vox, :])
        #wfit_z[vox, :] = np.interp(ppms, ppms - wppmoff[vox], wfit_z[vox, :])
        
    acq_mean = acq_z.mean(axis=0)
    fit_mean = fit_z.mean(axis=0)
    wfit_mean = wfit_z.mean(axis=0)
    lda_mean = (wfit_z - acq_z).mean(axis=0)
    
    acq_means[i, :] = acq_mean
    fit_means[i, :] = fit_mean
    wfit_means[i, :] = wfit_mean
    lda_means[i, :] = lda_mean
    
    residuals[i, :] = acq_mean - fit_mean
    
#%% plot
plt.close('all')

showvials = numvials

f, axarr = plt.subplots(2,4)
axarr = axarr.flatten()

#subplots:
#
for i in range(showvials):
    R_sq = round(np.corrcoef(acq_means[i,:], fit_means[i,:])[0,1]**2, 4)
    
    ax = axarr[i]
    ax.plot(ppms, fit_means.T[:,i], 'c-', lw=3, label='3-pool fit')
    ax.plot(ppms, acq_means.T[:,i], 'kd', ms=2, label='acquired data')
    ax.plot(ppms, residuals.T[:,i], 'k-', lw=1, label='residual')
    ax.plot([ppms[0], ppms[-1]], [0, 0], 'k:', lw=1)

    #ax.plot(ppms, wfit_means.T[:,i], 'r-', lw=1, label='wfit')    
    #ax.plot(ppms, lda_means.T[:,i], '-', label='wfit-acq')
    
    #ax.plot(ppms, residuals.T[:,i] + 0.8, 'k-', lw=1, label='residual (dotted=0)')
    #ax.plot([ppms[0], ppms[-1]], np.array([0, 0]) +0.8, 'k:', lw=1)   
    
    ax.set_xlim([-7.5, 7.5])
    ax.set_ylim([-0.16, 1.1])
    
    #for poolmarker in [2.8,-3.5]:
    #    ax.plot([poolmarker]*2,[-1,2], 'm:',lw=0.5)
    
    ax.set_xlabel('offset frequency (ppm)')
    ax.set_ylabel('$M_z$/$M^0$')
    ax.set_title(f'pH {vial_ph[i]}   $R^2$={R_sq}')
    ax.invert_xaxis()
    #for tick in ax.get_xticklabels(): tick.set_rotation(45)
    
handles, labels = ax.get_legend_handles_labels()
f.legend(handles, labels)

#LDA:
#
clrs = ['#0023ff', '#5900ff', '#7c00ff', '#9100ff', '#c000ff', '#eb00ff', '#ff00a4']
clrs = ['r', 'g', 'b', 'c', 'm', 'y', 'k']
f, ax = plt.subplots()

for i in range(showvials):
    ax.plot(ppms, lda_means[i,:], '-', label = f'pH {vial_ph[i]}',
            lw = 0.5+0.25*i, c=clrs[i])

ax.plot([-500, 500], [0, 0], 'k-', lw=1)
ax.set_xlim([-10, 10])
#ax.set_ylim([-0.01, 0.175])

#ax.plot([2.8,2.8], [0.03, 0.15], 'k--')
#ax.plot([-3.5,-3.5], [0.03, 0.15], 'k--')

for tick in ax.get_xticklabels(): tick.set_rotation(45)

ax.set_xlabel('offset frequency (ppm)')
ax.set_ylabel('$M_{water}/M^0-M_z/M^0$')
ax.invert_xaxis()
ax.legend()
f.tight_layout()

#%% cestr* plot
means = np.zeros([2,numvials])
cis = np.zeros_like(means)

for pidx, pool in enumerate(['b','c']):
    for i in vialnums:
        rstar = f'phan/NB{i}_std/CESTRstar_{pool}'
        mask = f'phan/masks/NB{i}'
        
        mean = fslstats(rstar, f'-k {mask} -M')[0]
        n = fslstats(rstar, f'-k {mask} -V')[0]
        sd = fslstats(rstar, f'-k {mask} -S')[0]
        ci = 1.96*sd/np.sqrt(n)
        
        means[pidx, i-1] = mean
        cis[pidx, i-1] = ci

slope, intcpt, r_value, p_value, std_err = stats.linregress(vial_ph, means[1,:])
print('linear regression', 'pearson r:', r_value, 'p:', p_value)
#print( stats.pearsonr(vial_ph, means[1,:]) )

f,ax = plt.subplots()
#ax.errorbar(vial_ph, means[0,:], yerr=cis[0,:], capsize=2, marker='s', ls='none', label='vial AmineR*')
ax.errorbar(vial_ph, means[1,:], yerr=cis[1,:], capsize=2, marker='s', ls='none', label='vial NOER*')
ax.plot(vial_ph, slope*np.array(vial_ph)+intcpt, 'k:', label='NOER* linear fit', lw=1)
ax.set_xlabel('pH')
ax.set_ylabel('CESTR*')
ax.legend(loc='upper left')



